#!/usr/bin/python

from gtk import *

def Run(*args):
  mainloop()

def Quit(*args):
  mainquit()

class ButtonChoiceWindow:
  def __init__(self):
    pass

  def __call__(self, title, text, buttons, default=0, long=0,
               transient=None):
    self._clicked = None

    win = GtkWindow()
    if transient:
      win.set_transient_for(transient)
    win.connect("destroy", mainquit)
    win.set_position(WIN_POS_CENTER)
    win.set_modal(1)
    win.set_title(title)
    win.set_border_width(5)

    v = GtkVBox()

    #if len(text) < 300:
    if not long:

      l = GtkLabel(text)
      l.set_line_wrap(1)
      l.show()
      v.add(l)

    else:  #if len(text) > 300:

      lbox = GtkHBox()
      l = GtkText(None, None)
      l.set_word_wrap(1)
      l.set_line_wrap(1)
      scroll = GtkVScrollbar(l.get_vadjustment())
      lbox.add(l, fill=1, expand=1)
      lbox.add(scroll, fill=0, expand=0)
      l.insert(None, None, None, text)
      #l.set_usize(400, 300)
      v.add(lbox, fill=1, expand=1)


    box = GtkHButtonBox()
    box.set_border_width(10)


    buttonlist = []
    i = 0
    for item in buttons:
      if type(item) == type((1,2)):
        label, value = item
      else:
        label = value = item


      b = GtkButton(label)
      buttonlist.append([label, value, b])
      b.connect("clicked", self._cb)

      if default >= 0  and  i == default:
        b.set_flags(CAN_DEFAULT)
        b.set_flags(HAS_DEFAULT)
        box.add(b)
      else:
        f = GtkFrame()
        f.set_border_width(2)
        f.set_shadow_type(SHADOW_NONE)
        f.add(b)
        box.add(f)

      i = i + 1

    box.show()
    v.add(box, fill=0, expand=0)

    v.show()
    win.add(v)

    win.show_all()
    mainloop()

    retval = None

    for item in buttonlist:
      label, value, b = item
      if b == self._clicked:
        retval = value

    win.hide()
    win.unrealize()
    del win

    return retval

  def _cb(self, button):
    self._clicked = button
    mainquit()

def EntryWindow(title, text, entry, buttons = ["Ok", "Cancel"], hidden=0, transient=None):
  win = Window(title)
  if transient:
    win.set_transient_for(transient)
  #win.set_modal(1)

  l = Label(text)
  l.set_padding(5)
  win.add(l)

  if type(entry) == type("Hi"):
    e = Entry("", entry, hidden=hidden)
  elif type(entry) == type((1,2)):
    label, text = entry
    e = Entry(text, label, hidden=hidden)
  e.set_padding(5)
  win.add(e)

  bbar = ButtonBar(buttons)
  bbar.set_padding(5)
  win.add(bbar)
  

  win.show()
  Run()
  win.hide()

  retval = None

  entered = e.get_value()
  clicked = bbar.get_value()

  retval = (clicked, entered)

  return retval

class Widget:
  def __init__(self):
    self._init()

  def _init(self):
    self._cb = {}
    self._cb_func = {
      "pressed" : self._pressed_cb,
      "released" : self._released_cb,
      "clicked" : self._clicked_cb,
      "toggled" : self._toggled_cb,
      "enter" : self._enter_cb,
      "leave" : self._leave_cb,
      "destroy" : self._destroy_cb,
      "" : None
      }


  def _pressed_cb(self, *args):
    self._callback("pressed")

  def _released_cb(self, *args):
    self._callback("released")

  def _clicked_cb(self, *args):
    self._callback("clicked")

  def _toggled_cb(self, *args):
    global _object_table
    (widget, ) = args
    value = widget.get_active()
    self._callback("toggled", value)

  def _enter_cb(self, *args):
    self._callback("enter")

  def _leave_cb(self, *args):
    self._callback("leave")

  def _destroy_cb(self, *args):
    self._callback("destroy")

  def _callback(self, event, *args):
    if self._cb.has_key(event):
      parameters = (self, event) + args
      #print "Calling %s cb...  Args: %s" % (event, repr(parameters))
      apply(self._cb[event], parameters)
      #print "Calling %s cb... done.\n" % event
  
  def set_cb(self, event, cb):
    if self._cb_func.has_key(event)  and  hasattr(self, "w"):
      self._cb[event] = cb
      self.w.connect(event, self._cb_func[event])
    else:
      raise ValueError, "Invalid event type %s" % event

  def run(self):
    Run()

  def show(self):
    if hasattr(self, "w"):
      self.w.show()
    
  def hide(self):
    if hasattr(self, "w"):
      self.w.hide()

  def set_padding(self, xpad, ypad=None):
    if ypad == None:
      ypad = xpad
    if hasattr(self, "w"):
      if hasattr(self.w, "set_padding"):
        self.w.set_padding(xpad, ypad)
      elif hasattr(self.w, "set_border_width"):
        self.w.set_border_width(xpad)

class Label(Widget):
  def __init__(self, text=""):
    self._init()
    self.w = GtkLabel(text)
    self.w.set_line_wrap(1)

  def set_text(self, text):
    self.w.set_text(text)

class Entry(Widget):
  def __init__(self, text=None, label=None, hidden=0):
    self._init()
    self.w = GtkHBox()
    self.e = GtkEntry()
    self.l = None
    if label:
      self.l = GtkLabel(label)
      self.w.add(self.l)
    if text:
      self.set_text(text)
    self.w.add(self.e)
    if hidden:
      self._set_visible(0)

  def set_text(self, text):
    self.e.set_text(text)

  def _set_visible(self, visible):
    self.e.set_visibility(visible)

  def get_value(self):
    return self.e.get_text()

class Button(Widget):
  def __init__(self, text=None):
    self._init()
    self.w = GtkButton()
    if text:
      self.set_text(text)

  def set_text(self, text):
    if hasattr(self, "l"):
      self.l.set_text(text)
    else:
      self.l = GtkLabel()
      self.l.set_line_wrap(1)
      self.l.set_padding(5, 3)
      self.l.set_text(text)
      self.w.add(self.l)

class CheckButton(Button):
  def __init__(self, text=None, value=0):
    self._init()
    self.w = GtkCheckButton()
    if text:
      self.set_text(text)

    self.set_value(value)

  def set_value(self, value):
    self.w.set_active(value)

  def get_value(self):
    return self.w.get_active()

class ButtonBar(Widget):
  def __init__(self, buttons=[]):
    self._init()
    self.w = GtkHButtonBox()
    self.w.set_layout_default(BUTTONBOX_SPREAD)
    self.buttons = []
    self._clicked = None
    for button in buttons:
      if type(button) == type((1,2)):
        text, value = button
      else:
        text = value = button
      b = Button(text)
      b.set_cb("clicked", self._default_cb)
      self.buttons.append((b, value))
      self.w.add(b.w)

  def set_cb(self, which, cb):
    try:
      (button, value) = self.buttons[which]
    except:
      raise IndexError, "Button out of range"
    button.set_cb("clicked", cb)

  def _default_cb(self, object, event):
    for item in self.buttons:
      (button, value) = item
      if button == object:
        self._clicked = value
        #print "Buttonbar clicked: %s" % value
        Quit()

  def get_value(self, result=None):
    return self._clicked

class ListBox(Widget):
  def __init__(self, height=10):
    self._init()
    self.w = GtkScrolledWindow()
    self.w.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
    self.w.set_border_width(2)
    self.l = GtkCList(1)
    self.l.set_usize(200, 20 + (height * 12))
    self.items = []
    self.l.connect("select-row", self._item_select_cb)
    self.l.set_selection_mode(SELECTION_BROWSE)
    self.w.add(self.l)
    
  def add_item(self, item):
    if type(item) == type((1,2)):
      text, value = item
    else:
      text = value = item
    
    #l = GtkListItem(text)
    self.items.append((text, value, 0))
    #items = [l]
    self.l.insert(len(self.items), [text])

  def add(self, items):
    if type(items) == type([1, 2]):
      for item in items:
        self.add_item(item)
    else:
      self.add_item(items)

  def get_value(self):
    result = []
    for item in self.items:
      (text, value, on) = item
      if on:
        result.append(value)

    if len(result) > 0:
      return result[0]
    else:
      return None
    

  def _item_select_cb(self, clist, row, col, gdkevent):
    #(gtklist, gtkitem) = args
    #gtkitem.select()
    self.deselect_all()
    #self.select(row)
    (text, value, on) = self.items[row]
    self.items[row] = (text, value, 1)
    #for i in range(len(self.items)):
    #  (object, text, value, on) = self.items[i]
    #  if object == gtkitem:
    #    self.select(i)

  def default(self, which):
    self.deselect_all()
    self.select(which)

  def deselect_all(self):
    for i in range(len(self.items)):
      (text, value, on) = self.items[i]
      #object.deselect()
      #self.l.unselect_row(i, 0)
      self.items[i] = (text, value, 0)

  # TODO: Make sure selected item is actually visible  (scroll to it)
  def select(self, which):
    if which >= 0  and  which < len(self.items):
      (text, value, on) = self.items[which]
      #object.select()
      self.l.select_row(which, 0)
      self.items[which] = (text, value, 1)

class Window(Widget):
  def __init__(self, title="", modal=0):
    self._init()
    self.w = GtkWindow()
    self.w.set_position(WIN_POS_MOUSE)
    self.w.connect("destroy", mainquit)
    self.w.set_title(title)
    self.w.set_modal(modal)
    self.box = GtkTable(99,99)
    self.w.add(self.box)
    self._current = 0
    #self.w.set_border_width(5)

  def set_title(self, title):
    self.w.set_title(title)

  def set_modal(self, modal):
    self.w.set_modal(modal)

  def set_transient_for(self, t):
    self.w.set_transient_for(t)

  def show(self):
    self.w.show_all()

  def hide(self):
    self.w.hide()

  def add(self, object, x=-1, y=-1, expand=0, gravity=None, padding=1):
    if object  and  hasattr(object, "w"):
      #self.box.add(object.w)
      if x>=0  and  y>=0:
        if expand:
          self.box.attach(object.w, x, x+1, y, y+1)
        else:
          self.box.attach(object.w, x, x+1, y, y+1, xoptions=0, yoptions=0)
      else:
        self.box.attach(object.w, 0, 1, self._current, self._current+1)
        #                xoptions=0, yoptions=0)
        self._current = self._current + 1

  def run(self, *args):
    self.show()
    Run()

  def quit(self, *args):
    Quit()

